﻿local addon, addonData = ...

local function GetDistance(x, y)
    local startX, startY = UnitPosition("player")
    local dX = startX - x
    local dY = startY - y
    return (dX * dX + dY * dY) ^ 0.5
end

local function GetClosestTreasure()
    local dist = 99999999
    local closest = nil
    for k,v in pairs(addonData.treasures) do
        local dist2 = GetDistance(v[3], v[4])
        if dist2 < dist and v[7] ~= 0 and not IsQuestFlaggedCompleted(v[7]) then
            dist = dist2
            closest = v
        end
    end
    return closest, dist
end

local function ShowArrow(x, y, text)
    if DBM then
        DBM.Arrow:ShowRunTo(x,y,nil,5)
        if DBMArrow then
            if not DBMArrow.text:IsShown() then
                DBMArrow.text:Show()
            end
            DBMArrow.text:SetText(text)
        end
    end
end

local function HideArrow()
    if DBM then
        DBM.Arrow:Hide()
        if DBMArrow then
            DBMArrow.text:Hide()
        end
    end
end

local interval = 1

local function onUpdate(self,elapsed)
    local _, _, _, mapid = UnitPosition("player")
    if mapid ~= 1116 then return end
    interval = interval + elapsed
    if interval >= 1 then
        interval = 0
        local closest, dist = GetClosestTreasure()
        if not closest then return end
        ShowArrow(closest[3],closest[4], format("%s\n%.02f yd", closest[5], dist))
    end
end

addonData.frame = CreateFrame("Frame")
addonData.frame:SetScript("OnUpdate", onUpdate)
addonData.enabled = true

SLASH_DRTS1 = "/drts";

SlashCmdList["DRTS"] = function(msg, editBox)
    if msg:lower() == "on" then
        addonData.frame:SetScript("OnUpdate", onUpdate)
        addonData.enabled = true
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures enabled")
    elseif msg:lower() == "off" then
        addonData.frame:SetScript("OnUpdate", nil)
        addonData.enabled = false
        HideArrow()
        DEFAULT_CHAT_FRAME:AddMessage("DraenorTreasures disabled")
    else
        DEFAULT_CHAT_FRAME:AddMessage("Usage: /drts on|off")
    end
end

if DBM and DBMArrow then
    local text = DBMArrow:CreateFontString(nil, "OVERLAY", "GameTooltipText")
    text:SetWidth(150)
    text:SetHeight(30)
    text:SetPoint("CENTER", DBMArrow, "CENTER", 0, -35)
    text:SetTextColor(1, 1, 1, 1)
    text:SetJustifyH("CENTER")
    DBMArrow.text = text
end

local function WorldPosToScreenPos(x, y, locLeft, locRight, locTop, locBottom)
    local h, w = abs(locRight - locLeft), abs(locBottom - locTop);
    x = (locTop - x) / w;
    y = (locLeft - y) / h;
    return y, x;
end

local function TreasureMap_ResetPOI(button)
    button:SetWidth(32);
    button:SetHeight(32);
    button.Texture:SetWidth(16);
    button.Texture:SetHeight(16);
    button.Texture:SetPoint("CENTER", 0, 0);
    button.Texture:SetTexture("Interface\\Minimap\\POIIcons");

    button.specialPOIInfo = nil;
end

local function TreasureMap_CreatePOI(index)
    local button = CreateFrame("Button", "TreasureMapFramePOI"..index, WorldMapButton);
    button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    button:SetScript("OnEnter", WorldMapPOI_OnEnter);
    button:SetScript("OnLeave", WorldMapPOI_OnLeave);
    button:SetScript("OnClick", WorldMapPOI_OnClick);

    button.Texture = button:CreateTexture(button:GetName().."Texture", "BACKGROUND");

    TreasureMap_ResetPOI(button);
    return button;
end

local function DrawTreasurePOI()
    local worldMapID = GetCurrentMapAreaID();
    local mapID, areaID, displayMapID, locLeft, locRight, locTop, locBottom, minLevel, maxLevel, flags = GetAreaMapInfo(worldMapID);

    for k,v in pairs(addonData.treasures) do
        local treasureMapPOIName = "TreasureMapFramePOI"..k;
        local treasureMapPOI = _G[treasureMapPOIName] or TreasureMap_CreatePOI(k);
        if v[2] ~= 0 and v[2] == areaID then
            local x, y = WorldPosToScreenPos(v[3], v[4], locLeft, locRight, locTop, locBottom);
            local name = format("%s (%u, %.02f, %.02f)", v[5], v[1], x * 100, y * 100);
            local description = v[6] ~= "" and v[6] or "Draenor Treasure!";

            x = x * WorldMapButton:GetWidth();
            y = -y * WorldMapButton:GetHeight();
            treasureMapPOI:SetPoint("CENTER", "WorldMapButton", "TOPLEFT", x, y);

            TreasureMap_ResetPOI(treasureMapPOI);

            local x1, x2, y1, y2 = GetPOITextureCoords(197);

            _G[treasureMapPOIName.."Texture"]:SetTexCoord(x1, x2, y1, y2);

            treasureMapPOI.name = name;
            treasureMapPOI.description = description;
            treasureMapPOI.mapLinkID = nil;

            treasureMapPOI:Show();
        else
            treasureMapPOI:Hide();
        end
    end
end

local function MyWorldMapFrame_OnUpdate(self, e)
    WorldMapFrame_OnUpdate(self, e);
    if addonData.enabled then
        DrawTreasurePOI();
    end
end

WorldMapFrame:SetScript("OnUpdate", MyWorldMapFrame_OnUpdate);
